@@ -21,40 +21,36 @@
 */
 
 #define R_NO_REMAP
+
 #include "key_conv.h"
 
+#include <R_ext/Visibility.h>
+#include <ctype.h>
+#include <stdbool.h>
 #include <stdio.h>
 #include <string.h>
-#include <stdbool.h>
 
 #define CHRKEY_PREFIX_CHUNK "chunk:"
 #define CHRKEY_PREFIX_PLAIN "plain:"
 #define CHRKEY_PREFIX_ACTOR "actor:"
-#define CHRKEY_PREFIX_ACTOR_DIGEST_KEYS "acdig:" //Not sure about this one
+#define CHRKEY_PREFIX_ACTOR_DIGEST_KEYS "acdig:"  // Not sure about this one
 
 #define RAWKEY_PREFIX_ACTOR "actorprefix"
 #define RAWKEY_PREFIX_ACTOR_DIGEST_KEYS "digp"
 
-#define CHUNK_KEY_TAG_MIN 33
-#define CHUNK_KEY_TAG_MAX 96
-#define CHUNK_KEY_LEGACY_VERSION_TAG 118
-#define CHUNK_KEY_DIM_MAX 2
-#define CHUNK_KEY_SUBCHUNK_MIN -32
-#define CHUNK_KEY_SUBCHUNK_MAX 31
-
 enum KEY_TYPE {
-    PLAIN = 0,        // plain:~local_player
-    CHUNK,            // chunk:x:z:d:t:s
-    ACTOR,            // actor:0000000000000100
-    ACTOR_DIGEST_KEYS // acdat:x:z:d
+    PLAIN = 0,         // plain:~local_player
+    CHUNK,             // chunk:x:z:d:t:s
+    ACTOR,             // actor:0000000000000100
+    ACTOR_DIGEST_KEYS  // acdat:x:z:d
 };
 
 static char encode_hex(unsigned char x) {
     x = x & 15;
     if(x < 10) {
-        return (char)('0'+x);
+        return (char)('0' + x);
     } else {
-        return (char)('A'+x-10);
+        return (char)('A' + x - 10);
     }
 }
 
@@ -72,13 +68,13 @@ static unsigned char decode_hex_digit(char ch) {
 static size_t str_to_uint(const char *str, size_t len, unsigned int *out) {
     unsigned int val = 0;
     size_t i = 0;
-    for(;i<len;++i) {
+    for(; i < len; ++i) {
         char ch = str[i];
         if('0' > ch || ch > '9') {
             break;
         }
         ch = ch - '0';
-        val = val*10 + ch;
+        val = val * 10 + ch;
     }
     if(out != NULL) {
         *out = val;
@@ -94,8 +90,10 @@ static size_t str_to_int(const char *str, size_t len, int *out) {
     int val;
     unsigned int u;
     if(str[0] == '-') {
-        ret = str_to_uint(str+1, len-1, &u);
-        ret += 1;
+        ret = str_to_uint(str + 1, len - 1, &u);
+        if(ret != 0) {
+            ret += 1;
+        }
         val = -u;
     } else {
         ret = str_to_uint(str, len, &u);
@@ -112,15 +110,16 @@ static bool has_prefix(const char *key, size_t key_len, const char *prefix) {
     if(key_len < prefix_len) {
         return false;
     }
-    return(strncmp(key, prefix, prefix_len) == 0);
+    return (strncmp(key, prefix, prefix_len) == 0);
 }
 
 // Percent encode keys.
 // Writes up to buffer_len-1 characters into buffer.
 // Appends '\0' to written string.
-// Returns the total length of the encoding (minus terminating null), even if all
-// characters were not written.
-static size_t percent_encode(const unsigned char *key, size_t key_len, char *buffer, size_t buffer_len) {
+// Returns the total length of the encoding (minus terminating null), even if
+// all characters were not written.
+static size_t percent_encode(const unsigned char *key, size_t key_len,
+                             char *buffer, size_t buffer_len) {
     // Identify the beginning of the string up until the first
     // character that needs encoding. Most strings can be copied
     // as is.
@@ -137,7 +136,7 @@ static size_t percent_encode(const unsigned char *key, size_t key_len, char *buf
         if(buffer_len > ret_len) {
             memcpy(buffer, key, ret_len);
         } else if(buffer_len > 0) {
-            memcpy(buffer, key, buffer_len-1);
+            memcpy(buffer, key, buffer_len - 1);
         }
     }
     // encode the rest of the string
@@ -145,14 +144,14 @@ static size_t percent_encode(const unsigned char *key, size_t key_len, char *buf
         unsigned char ch = key[i];
 
         if(ch <= 0x20 || ch >= 0x7F || ch == '%') {
-            if(buffer_len > ret_len+3 ) {
+            if(buffer_len > ret_len + 3) {
                 buffer[ret_len] = '%';
-                buffer[ret_len+1] = encode_hex(ch >> 4);
-                buffer[ret_len+2] = encode_hex(ch & 0xF);
+                buffer[ret_len + 1] = encode_hex(ch >> 4);
+                buffer[ret_len + 2] = encode_hex(ch & 0xF);
             }
             ret_len += 3;
         } else {
-            if(buffer_len > ret_len+1 ) {
+            if(buffer_len > ret_len + 1) {
                 buffer[ret_len] = ch;
             }
             ret_len += 1;
@@ -161,7 +160,7 @@ static size_t percent_encode(const unsigned char *key, size_t key_len, char *buf
     if(buffer_len > ret_len) {
         buffer[ret_len] = '\0';
     } else if(buffer_len > 0) {
-        buffer[buffer_len-1] = '\0';
+        buffer[buffer_len - 1] = '\0';
     }
     return ret_len;
 }
@@ -170,7 +169,8 @@ static size_t percent_encode(const unsigned char *key, size_t key_len, char *buf
 // Writes up to buffer_len characters into buffer.
 // Returns the total length of the encoding, even if all
 // characters were not written.
-static size_t percent_decode(const char *key, size_t key_len, unsigned char *buffer, size_t buffer_len) {
+static size_t percent_decode(const char *key, size_t key_len,
+                             unsigned char *buffer, size_t buffer_len) {
     size_t i = 0;
     for(; i < key_len; ++i) {
         if(key[i] == '%') {
@@ -189,11 +189,11 @@ static size_t percent_decode(const char *key, size_t key_len, unsigned char *buf
     // decode the rest of the string
     for(; i != key_len; ++i) {
         unsigned char ch = key[i];
-        if(ch == '%' && i+2 < key_len) {
-            unsigned char a = decode_hex_digit(key[i+1]);
-            unsigned char b = decode_hex_digit(key[i+2]);
-            if(a < 16  && b < 16) {
-                ch = (unsigned char)(a*16+b);
+        if(ch == '%' && i + 2 < key_len) {
+            unsigned char a = decode_hex_digit(key[i + 1]);
+            unsigned char b = decode_hex_digit(key[i + 2]);
+            if(a < 16 && b < 16) {
+                ch = (unsigned char)(a * 16 + b);
                 i += 2;
             }
         }
@@ -207,45 +207,43 @@ static size_t percent_decode(const char *key, size_t key_len, unsigned char *buf
 
 // decode a prefix in the format of x:z:d
 //  returns how many bytes from key were read.
-size_t decode_chunk_prefix(const char *key, size_t key_len, int *x, int *z, unsigned int *dimension) {
+size_t decode_chunk_prefix(const char *key, size_t key_len, int *x, int *z,
+                           int *dimension) {
     size_t sz = 0;
     size_t i = 0;
 
     // decode x
-    sz = str_to_int(key+i, key_len-i, x);
+    sz = str_to_int(key + i, key_len - i, x);
     i += sz;
     if(sz == 0 || i == key_len || key[i] != ':') {
         return 0;
     }
     i += 1;
     // decode z
-    sz = str_to_int(key+i, key_len-i, z);
+    sz = str_to_int(key + i, key_len - i, z);
     i += sz;
     if(sz == 0 || i == key_len || key[i] != ':') {
         return 0;
     }
     i += 1;
     // decode dimension
-    sz = str_to_uint(key+i, key_len-i, dimension);
+    sz = str_to_int(key + i, key_len - i, dimension);
     i += sz;
     if(sz == 0) {
         return 0;
     }
-    // validate dimension
-    if(*dimension > CHUNK_KEY_DIM_MAX) {
-        return 0;
-    }
     return i;
 }
 
 // decode x:z:dimension:tag and x:z:dimension:tag:subtag
 // writes decoding into buffer and returns the number of byte written
 // if buffer is too small, it returns the number of bytes that would be written
-size_t chunkkey_decode(const char *key, size_t key_len, unsigned char *buffer, size_t buffer_len) {
+size_t chunkkey_decode(const char *key, size_t key_len, unsigned char *buffer,
+                       size_t buffer_len) {
     unsigned int u = 0;
     int d = 0;
     int x = 0, z = 0;
-    unsigned int dimension = 0;
+    int dimension = 0;
     signed char tag = 0;
     signed char subtag = 0;
     bool has_subtag = false;
@@ -261,7 +259,7 @@ size_t chunkkey_decode(const char *key, size_t key_len, unsigned char *buffer, s
     }
     i += 1;
     // decode tag
-    sz = str_to_uint(key+i, key_len-i, &u);
+    sz = str_to_uint(key + i, key_len - i, &u);
     if(sz == 0) {
         return 0;
     }
@@ -273,7 +271,7 @@ size_t chunkkey_decode(const char *key, size_t key_len, unsigned char *buffer, s
             return 0;
         }
         i += 1;
-        sz = str_to_int(key+i, key_len-i, &d);
+        sz = str_to_int(key + i, key_len - i, &d);
         i += sz;
         if(sz == 0 || i != key_len) {
             return 0;
@@ -281,33 +279,25 @@ size_t chunkkey_decode(const char *key, size_t key_len, unsigned char *buffer, s
         subtag = (signed char)d;
         has_subtag = true;
     }
-    // Validate values
-    if(tag < CHUNK_KEY_TAG_MIN) {
-        return 0;
-    } else if(tag > CHUNK_KEY_TAG_MAX && tag != CHUNK_KEY_LEGACY_VERSION_TAG) {
-        return 0;
-    } else if(subtag < CHUNK_KEY_SUBCHUNK_MIN || subtag > CHUNK_KEY_SUBCHUNK_MAX) {
-        return 0;
-    }
 
     // Check buffer space
-    size_t decode_len = 8+4*(dimension != 0)+1+(has_subtag);
+    size_t decode_len = 8 + 4 * (dimension != 0) + 1 + (has_subtag);
     if(buffer_len < decode_len) {
-        return decode_len; // # nocov
+        return decode_len;  // # nocov
     }
     // Write values
     i = 0;
-    memcpy(buffer+i,&x,4);
+    memcpy(buffer + i, &x, 4);
     i += 4;
-    memcpy(buffer+i,&z,4);
+    memcpy(buffer + i, &z, 4);
     i += 4;
-    if(dimension > 0) {
-        memcpy(buffer+i,&dimension,4);
+    if(dimension != 0) {
+        memcpy(buffer + i, &dimension, 4);
         i += 4;
     }
     buffer[i] = tag;
     if(has_subtag) {
-        buffer[i+1] = subtag;
+        buffer[i + 1] = subtag;
     }
     return decode_len;
 }
@@ -315,9 +305,10 @@ size_t chunkkey_decode(const char *key, size_t key_len, unsigned char *buffer, s
 // decode x:z:dimension
 // writes decoding into buffer and returns the number of byte written
 // if buffer is too small, it returns the number of bytes that would be written
-size_t digkey_decode(const char *key, size_t key_len, unsigned char *buffer, size_t buffer_len) {
+size_t digkey_decode(const char *key, size_t key_len, unsigned char *buffer,
+                     size_t buffer_len) {
     int x = 0, z = 0;
-    unsigned int dimension = 0;
+    int dimension = 0;
 
     // decode prefix
     size_t sz = decode_chunk_prefix(key, key_len, &x, &z, &dimension);
@@ -326,56 +317,55 @@ size_t digkey_decode(const char *key, size_t key_len, unsigned char *buffer, siz
     }
 
     const size_t prefix_len = strlen(RAWKEY_PREFIX_ACTOR_DIGEST_KEYS);
-    size_t decode_len = prefix_len+8+4*(dimension != 0);
+    size_t decode_len = prefix_len + 8 + 4 * (dimension != 0);
     if(buffer_len < decode_len) {
-        return decode_len; // # nocov
+        return decode_len;  // # nocov
     }
-    size_t i=0;
+    size_t i = 0;
     // prefix
     memcpy(buffer, RAWKEY_PREFIX_ACTOR_DIGEST_KEYS, prefix_len);
     i += prefix_len;
     // x
-    memcpy(buffer+i,&x,4);
+    memcpy(buffer + i, &x, 4);
     i += 4;
     // z
-    memcpy(buffer+i,&z,4);
+    memcpy(buffer + i, &z, 4);
     i += 4;
     // dimension
-    if(dimension > 0) {
-        memcpy(buffer+i,&dimension,4);
+    if(dimension != 0) {
+        memcpy(buffer + i, &dimension, 4);
         /* i += 4; */
     }
 
     return decode_len;
 }
 
-
 // decode aabbccddeeffgghh
 // writes decoding into buffer and returns the number of byte written
 // if buffer is too small, it returns the number of bytes that would be written
-size_t actorkey_decode(const char *key, size_t key_len, unsigned char *buffer, size_t buffer_len) {
-
+size_t actorkey_decode(const char *key, size_t key_len, unsigned char *buffer,
+                       size_t buffer_len) {
     // validate key
     if(key_len != 16) {
         return 0;
     }
-    for(int i=0;i<16;++i) {
+    for(int i = 0; i < 16; ++i) {
         unsigned char a = decode_hex_digit(key[i]);
         if(a >= 16) {
             return 0;
         }
     }
     const size_t prefix_len = strlen(RAWKEY_PREFIX_ACTOR);
-    size_t decode_len = prefix_len+8;
+    size_t decode_len = prefix_len + 8;
     if(buffer_len < decode_len) {
-        return decode_len; // # nocov
+        return decode_len;  // # nocov
     }
     memcpy(buffer, RAWKEY_PREFIX_ACTOR, prefix_len);
-    for(int i=0;i<16;i+=2) {
+    for(int i = 0; i < 16; i += 2) {
         unsigned char a = decode_hex_digit(key[i]);
-        unsigned char b = decode_hex_digit(key[i+1]);
-        unsigned char ch = (unsigned char)(a*16+b);
-        buffer[prefix_len+i/2] = ch;
+        unsigned char b = decode_hex_digit(key[i + 1]);
+        unsigned char ch = (unsigned char)(a * 16 + b);
+        buffer[prefix_len + i / 2] = ch;
     }
 
     return decode_len;
@@ -384,10 +374,11 @@ size_t actorkey_decode(const char *key, size_t key_len, unsigned char *buffer, s
 // everything else is percent encoded.
 // Writes up to buffer_len-1 characters into buffer.
 // Appends '\0' to written string.
-// Returns the total length of the encoding (minus terminating null), even if all
-// characters were not written.
-size_t rawkey_to_chrkey(const unsigned char *key, size_t key_len, char *buffer, size_t buffer_len) {
-    unsigned int dimension = 0;
+// Returns the total length of the encoding (minus terminating null), even if
+// all characters were not written.
+size_t rawkey_to_chrkey(const unsigned char *key, size_t key_len, char *buffer,
+                        size_t buffer_len) {
+    int dimension = 0;
     int x = 0;
     int z = 0;
     signed char tag = 0;
@@ -398,56 +389,60 @@ size_t rawkey_to_chrkey(const unsigned char *key, size_t key_len, char *buffer,
 
     // Detect key type
     switch(key_len) {
-     case 10:
+    case 10:
         subtag = key[9];
         has_subtag = true;
-     case  9:
+    case 9:
         tag = key[8];
         key_type = CHUNK;
         break;
-     case 14:
+    case 14:
         subtag = key[13];
         has_subtag = true;
-     case 13:
+    case 13:
         tag = key[12];
         key_type = CHUNK;
         break;
-     default:
-        {
-            if(has_prefix((char *)key, key_len, RAWKEY_PREFIX_ACTOR)) {
-                key_type = ACTOR;
-            } else if(has_prefix((char *)key, key_len, RAWKEY_PREFIX_ACTOR_DIGEST_KEYS)) {
-                key_type = ACTOR_DIGEST_KEYS;
-            } else {
-                key_type = PLAIN;
-            }
-            break;
+    default: {
+        if(has_prefix((char *)key, key_len, RAWKEY_PREFIX_ACTOR)) {
+            key_type = ACTOR;
+        } else if(has_prefix((char *)key, key_len,
+                             RAWKEY_PREFIX_ACTOR_DIGEST_KEYS)) {
+            key_type = ACTOR_DIGEST_KEYS;
+        } else {
+            key_type = PLAIN;
         }
+        break;
+    }
     }
     // validate data
     if(key_type == CHUNK) {
-        if(tag < CHUNK_KEY_TAG_MIN) {
-            key_type = PLAIN;
-        } else if(tag > CHUNK_KEY_TAG_MAX && tag != CHUNK_KEY_LEGACY_VERSION_TAG) {
-            key_type = PLAIN;
-        } else if(subtag < CHUNK_KEY_SUBCHUNK_MIN || subtag > CHUNK_KEY_SUBCHUNK_MAX) {
+        // If a "chunk" key contains only printable characters, mark it as
+        // a plain key.
+        size_t i = 0;
+        for(; i < key_len; ++i) {
+            if(!isprint(key[i])) {
+                break;
+            }
+        }
+        if(i == key_len) {
             key_type = PLAIN;
         }
     } else if(key_type == ACTOR_DIGEST_KEYS) {
         size_t len = strlen(RAWKEY_PREFIX_ACTOR_DIGEST_KEYS);
-        if(key_len != len+8 && key_len != len+12) {
+        if(key_len != len + 8 && key_len != len + 12) {
             key_type = PLAIN;
         }
     } else if(key_type == ACTOR) {
         size_t len = strlen(RAWKEY_PREFIX_ACTOR);
-        if(key_len != len+8) {
+        if(key_len != len + 8) {
             key_type = PLAIN;
         }
     }
 
     // extract chunk_prefix coordinates
     if(key_type == CHUNK || key_type == ACTOR_DIGEST_KEYS) {
-        const unsigned char * p = key;
+        const unsigned char *p = key;
         size_t len = key_len;
         if(key_type == ACTOR_DIGEST_KEYS) {
             p += strlen(RAWKEY_PREFIX_ACTOR_DIGEST_KEYS);
@@ -457,29 +452,27 @@ size_t rawkey_to_chrkey(const unsigned char *key, size_t key_len, char *buffer,
         memcpy(&z, p + 4, 4);
         if(len >= 12) {
             memcpy(&dimension, p + 8, 4);
-            // validate dimension
-            if(dimension > CHUNK_KEY_DIM_MAX) {
-                key_type = PLAIN;
-            }
         }
     }
 
     if(key_type == CHUNK && has_subtag) {
-        return snprintf(buffer, buffer_len, CHRKEY_PREFIX_CHUNK "%d:%d:%u:%u:%d", x, z, dimension,
-            (unsigned int)tag, (int)subtag);
+        return snprintf(buffer, buffer_len,
+                        CHRKEY_PREFIX_CHUNK "%d:%d:%d:%u:%d", x, z, dimension,
+                        (unsigned int)tag, (int)subtag);
     } else if(key_type == CHUNK) {
-        return snprintf(buffer, buffer_len, CHRKEY_PREFIX_CHUNK "%d:%d:%u:%u", x, z, dimension,
-            (unsigned int)tag);
+        return snprintf(buffer, buffer_len, CHRKEY_PREFIX_CHUNK "%d:%d:%d:%u",
+                        x, z, dimension, (unsigned int)tag);
     } else if(key_type == ACTOR_DIGEST_KEYS) {
         return snprintf(buffer, buffer_len,
-                CHRKEY_PREFIX_ACTOR_DIGEST_KEYS "%d:%d:%u", x, z, dimension);
+                        CHRKEY_PREFIX_ACTOR_DIGEST_KEYS "%d:%d:%d", x, z,
+                        dimension);
     } else if(key_type == ACTOR) {
         key = key + strlen(RAWKEY_PREFIX_ACTOR);
         size_t prefix_len = strlen(CHRKEY_PREFIX_ACTOR);
-        if(prefix_len+2*8+1 < buffer_len) {
-            memcpy(buffer, (char*)CHRKEY_PREFIX_ACTOR, prefix_len);
+        if(prefix_len + 2 * 8 + 1 < buffer_len) {
+            memcpy(buffer, (char *)CHRKEY_PREFIX_ACTOR, prefix_len);
             buffer += prefix_len;
-            for(int i=0; i < 8; ++i) {
+            for(int i = 0; i < 8; ++i) {
                 unsigned char ch = key[i];
                 buffer[0] = encode_hex(ch >> 4);
                 buffer[1] = encode_hex(ch & 0xF);
@@ -487,7 +480,7 @@ size_t rawkey_to_chrkey(const unsigned char *key, size_t key_len, char *buffer,
             }
             buffer[0] = '\0';
         }
-        return prefix_len+2*8;
+        return prefix_len + 2 * 8;
     }
     // Plain keys
     size_t prefix_len = strlen(CHRKEY_PREFIX_PLAIN);
@@ -496,13 +489,13 @@ size_t rawkey_to_chrkey(const unsigned char *key, size_t key_len, char *buffer,
         buffer += prefix_len;
         buffer_len -= prefix_len;
     } else {
-        buffer_len = 0; // # nocov
+        buffer_len = 0;  // # nocov
     }
     return prefix_len + percent_encode(key, key_len, buffer, buffer_len);
 }
 
 // Take a VECSXP of raw, internal keys and covert them to human-readable keys.
-SEXP rawkeys_to_chrkeys(SEXP r_keys) {
+SEXP attribute_visible R_rawkeys_to_chrkeys(SEXP r_keys) {
     // Maximum chunk data key size is 45 = 1+11+1+11+1+10+1+3+1+4+1
     // Maximum other key size is 3*len.
     char buffer[2048];
@@ -524,13 +517,18 @@ SEXP rawkeys_to_chrkeys(SEXP r_keys) {
             continue;
         }
         if(TYPEOF(elm) != RAWSXP) {
-            Rf_error("Element %td of argument 'key' is not a raw type or NULL.", i);
+            Rf_error("Element %td of argument 'key' is not a raw type or NULL.",
+                     i);
             return R_NilValue;
         }
         R_xlen_t raw_len = XLENGTH(elm);
-        size_t key_len = rawkey_to_chrkey(RAW(elm), raw_len, buffer, buffer_len);
+        size_t key_len =
+            rawkey_to_chrkey(RAW(elm), raw_len, buffer, buffer_len);
         if(key_len >= buffer_len) {
-            Rf_error("Conversion of element %td of argument 'key' exceeded buffer space.", i+1);
+            Rf_error(
+                "Conversion of element %td of argument 'key' exceeded buffer "
+                "space.",
+                i + 1);
             return R_NilValue;
         }
         SET_STRING_ELT(r_ret, i, Rf_mkCharLenCE(buffer, (int)key_len, CE_UTF8));
@@ -543,7 +541,8 @@ SEXP rawkeys_to_chrkeys(SEXP r_keys) {
 // Writes up to buffer_len characters into buffer.
 // Returns the total length of the encoding, even if all
 // characters were not written.
-size_t chrkey_to_rawkey(const char *key, size_t key_len, unsigned char *buffer, size_t buffer_len) {
+static size_t chrkey_to_rawkey(const char *key, size_t key_len,
+                               unsigned char *buffer, size_t buffer_len) {
     if(key_len == 0) {
         return 0;
     }
@@ -555,17 +554,18 @@ size_t chrkey_to_rawkey(const char *key, size_t key_len, unsigned char *buffer,
     } else if(has_prefix(key, key_len, CHRKEY_PREFIX_CHUNK)) {
         size_t len = strlen(CHRKEY_PREFIX_CHUNK);
         key_type = CHUNK;
-        ret = chunkkey_decode(key+len, key_len-len, buffer, buffer_len);
+        ret = chunkkey_decode(key + len, key_len - len, buffer, buffer_len);
     } else if(has_prefix(key, key_len, CHRKEY_PREFIX_ACTOR)) {
         size_t len = strlen(CHRKEY_PREFIX_ACTOR);
         key_type = ACTOR;
-        ret = actorkey_decode(key+len, key_len-len, buffer, buffer_len);
+        ret = actorkey_decode(key + len, key_len - len, buffer, buffer_len);
     } else if(has_prefix(key, key_len, CHRKEY_PREFIX_ACTOR_DIGEST_KEYS)) {
         size_t len = strlen(CHRKEY_PREFIX_ACTOR_DIGEST_KEYS);
         key_type = ACTOR_DIGEST_KEYS;
-        ret = digkey_decode(key+len, key_len-len, buffer, buffer_len);
+        ret = digkey_decode(key + len, key_len - len, buffer, buffer_len);
     } else {
-        Rf_warning("Unknown or missing prefix in key; assuming key is a plain key.");
+        Rf_warning(
+            "Unknown or missing prefix in key; assuming key is a plain key.");
     }
 
     // check to see if key has been decoded, otherwise assume plain key
@@ -579,15 +579,17 @@ size_t chrkey_to_rawkey(const char *key, size_t key_len, unsigned char *buffer,
     } else if(key_type == ACTOR) {
         Rf_warning("Invalid actor key format; assuming key is a plain key.");
     } else if(key_type == ACTOR_DIGEST_KEYS) {
-        Rf_warning("Invalid actor digest keys key format; assuming key is a plain key.");
+        Rf_warning(
+            "Invalid actor digest keys key format; assuming key is a plain "
+            "key.");
     }
 
-    return percent_decode(key, key_len, buffer, buffer_len);    
+    return percent_decode(key, key_len, buffer, buffer_len);
 }
 
-SEXP chrkeys_to_rawkeys(SEXP r_keys) {
+SEXP attribute_visible R_chrkeys_to_rawkeys(SEXP r_keys) {
     unsigned char buffer[2048];
-    const int buffer_len = 2048;    
+    const int buffer_len = 2048;
 
     if(Rf_isNull(r_keys)) {
         return R_NilValue;
@@ -604,9 +606,13 @@ SEXP chrkeys_to_rawkeys(SEXP r_keys) {
             SET_VECTOR_ELT(r_ret, i, R_NilValue);
             continue;
         }
-        size_t raw_len = chrkey_to_rawkey(CHAR(elm), XLENGTH(elm), buffer, buffer_len);
+        size_t raw_len =
+            chrkey_to_rawkey(CHAR(elm), XLENGTH(elm), buffer, buffer_len);
         if(raw_len > buffer_len) {
-            Rf_error("Conversion of element %td of argument 'key' exceeded buffer space.", i+1);
+            Rf_error(
+                "Conversion of element %td of argument 'key' exceeded buffer "
+                "space.",
+                i + 1);
             return R_NilValue;
         }
         SET_VECTOR_ELT(r_ret, i, Rf_allocVector(RAWSXP, raw_len));
